/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic
 * Semiconductor ASA.Terms and conditions of usage are described in detail
 * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 133 $
 */

/* 
 * This is an example of how to program the infopage of a LE1 from another LE1. 
 * Set the variable new_infopage to the page you want programmed, and run the code. 
 * 
 * The program uses UART as UI, so make sure you have turned on S11, and 
 * connected the serial port on the motherboard to the computer. The connection 
 * is 57,6 kbps, 8 bit data, 1 stop bit. 
 *
 * This program will run correctly only on the QFN32 LE1. On QFN24 there isn't 
 * enough GPIO and on QFN48, the master SPI overlaps the UART.
 * 
 * The pacakage size you want to write to, must be defined below. 
 */
//#define QFN24
//#define QFN32
//#define QFN48

#define SPI_CMD_WREN 0x06
#define SPI_CMD_RDSR 0x05
#define SPI_CMD_WRSR 0x01
#define SPI_CMD_READ 0x03
#define SPI_CMD_PROGRAM 0x02
#define SPI_CMD_ERASE_PAGE 0x52

#define INFOPAGE_MEMORY_ADDRESS 0x0000
#define INFOPAGE_PAGE_NUMBER 0x00

#define SLAVE_CSN P12
#define SLAVE_PROG P11
#define SLAVE_RESET P10

#include <stdio.h>
#include <nrf24le1.h>
#include <hal_uart.h>
#include <hal_clk.h>
#include <hal_delay.h>
#include <hal_spi.h>

// This is the default infopages for the different nRF24LE1s
#ifdef QFN24
uint8_t new_infopage[32] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 
                            0x21, 0xbf, 0x50, 0x15, 0x10, 0xff, 0xff, 0xff, 0xff, 0x82, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
#elif defined QFN32
uint8_t new_infopage[32] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 
                            0x26, 0xbf, 0x50, 0x15, 0x10, 0xff, 0xff, 0xff, 0xff, 0x82, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
#elif defined QFN48
uint8_t new_infopage[32] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 
                            0x23, 0xbf, 0x50, 0x15, 0x10, 0xff, 0xff, 0xff, 0xff, 0x82, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
#else
#error No package size defined. Defined either QFN24, QFN32 or QFN48.   
uint8_t new_infopage[32];                        
#endif                           

uint8_t current_infopage[32];

char putchar(char c) {
  hal_uart_putchar(c);
  return c;
}

char getchar(void) {
  return hal_uart_getchar();
}


void enable_flash_writing(void) {
  SLAVE_CSN = 0;
  hal_spi_master_read_write(SPI_CMD_WREN);
  SLAVE_CSN = 1;
}

void print_infopage(uint8_t *ip) {
  uint8_t i;
  printf("Infopage: \r\n0x00:\t");
  for(i = 0; i < 32; i++) {
    printf("0x%02bx ", ip[i]);
    if(i == 15) printf("\r\n0x10:\t");
  }      
  printf("\r\n");
}

void read_infopage(uint8_t *ip) {
  uint8_t i = 0; 
  
  SLAVE_CSN = 0;
  hal_spi_master_read_write(SPI_CMD_READ);
  hal_spi_master_read_write(INFOPAGE_MEMORY_ADDRESS);
  hal_spi_master_read_write(INFOPAGE_MEMORY_ADDRESS << 8);
  for(i = 0; i < 32; i++)
    ip[i] = hal_spi_master_read_write(0x00);
  SLAVE_CSN = 1;
}


void write_infopage(uint8_t *ip) {
  uint8_t i = 0; 
  
  enable_flash_writing();

  // Erase infopage  
  SLAVE_CSN = 0;
  hal_spi_master_read_write(SPI_CMD_ERASE_PAGE);
  hal_spi_master_read_write(INFOPAGE_PAGE_NUMBER);
  SLAVE_CSN = 1;
  
  delay_ms(100);
  
  // Writing is disabled when erasing. Enable it again.
  enable_flash_writing();
  
  // Do the actual writing
  SLAVE_CSN = 0;
  hal_spi_master_read_write(SPI_CMD_PROGRAM);
  hal_spi_master_read_write(INFOPAGE_MEMORY_ADDRESS);
  hal_spi_master_read_write(INFOPAGE_MEMORY_ADDRESS << 8);
  for(i = 0; i < 32; i++)
    hal_spi_master_read_write(ip[i]);
  SLAVE_CSN = 1;

}

void main(void)
{
  uint8_t i = 0;
  char cmd;
  // Set TXD pin as output.
  P0DIR = 0xF7;
  
  // Set PROG, RESET and CSN as outputs, and to good default states.
  P1DIR = 0xC8;
  SLAVE_PROG = 0;
  SLAVE_RESET = 1;
  SLAVE_CSN = 1;

   // Initializes the UART
  hal_uart_init(UART_BAUD_57K6);

  // Wait for XOSC to start to ensure proper UART baudrate
  while(hal_clk_get_16m_source() != HAL_CLK_XOSC16M);

  // Enable global interrupts
  EA = 1;

  printf("\r\nBoot\r\n");
  
  // Set the device in flash SPI slave mode, see chapter 6.3.5 of the specification
  delay_us(5);
  SLAVE_PROG = 1;
  delay_us(5);
  SLAVE_RESET = 0;
  delay_us(2);
  SLAVE_RESET = 1;
  
  // Initialize the SPI controller
  hal_spi_master_init(SPI_CLK_DIV64, HAL_SPI_MODE_0, HAL_SPI_MSB_LSB);
  
  // Switch to read infopage instead of code flash 
  SLAVE_CSN = 0;
  hal_spi_master_read_write(SPI_CMD_WRSR);
  hal_spi_master_read_write(0x28); // FSR.INFEN = 1, FSR.WEN = 1
  SLAVE_CSN = 1;
  
  // Read out the FSR register
  SLAVE_CSN = 0;
  hal_spi_master_read_write(SPI_CMD_RDSR);
  printf("FSR: 0x%bx\r\n", hal_spi_master_read_write(0x05));
  SLAVE_CSN = 1;
    
  for(;;) {
    read_infopage(&current_infopage);
    print_infopage(&current_infopage);
      
    printf("Press p to program new infopage, r to read infopage again or s to stop.\r\n");
    cmd = getchar();
    
    switch(cmd) {
      default:
        printf("Unknown command\r\n");
        // Fall through to stopping
        
      case 's':
        printf("Stopping. Reset to start again.\r\n");
        PWRDWN = 0x00;
        break;
      
      case 'p':
        printf("Programming...\r\n");
        write_infopage(&new_infopage);
        printf("Programming done.\r\n");
        
      case 'r':
        // Do nothing, fall through
        ;
    }
  }
}
/** @} */
